# Copyright 2015 Canonical Ltd.  This software is licensed under the
# GNU Affero General Public License version 3 (see the file LICENSE).

import os
import sqlite3
import unittest

from fixtures import EnvironmentVariable
from testtools import TestCase
from webtest import TestApp

import rutabaga


class ApiBaseTestCase(TestCase):

    tempdb = 'tempdb'

    def _db_conn(self):
        conn = sqlite3.connect(self.tempdb)
        with open('db/rutabaga.schema', 'r') as schema:
            conn.executescript(schema.read())
            conn.commit()
        return conn

    def setUp(self):
        super(ApiBaseTestCase, self).setUp()
        self.useFixture(EnvironmentVariable("DATABASE", self.tempdb))
        self.app = TestApp(rutabaga.main({}))
        self.db = self._db_conn()

    def tearDown(self):
        super(ApiBaseTestCase, self).tearDown()
        self.db.close()
        os.remove(self.tempdb)


class ApiTokenTestCase(ApiBaseTestCase):

    def test_create_token(self):
        username = 'BUILD-1234'
        resp = self.app.post_json('/tokens', {'username': username})
        self.assertIn(username, resp.json['username'])
        self.assertEqual(200, resp.status_code)

    def test_invalid_post(self):
        resp = self.app.post_json(
            '/tokens', {'junk': 'junk'}, expect_errors=True)
        self.assertIn('username is missing',
                      resp.json['errors'][0]['description'])
        self.assertEqual(400, resp.status_code)

    def test_invalid_delete(self):
        resp = self.app.delete('/tokens/', expect_errors=True)
        self.assertEqual(404, resp.status_code)

    def test_invalid_delete_token(self):
        resp = self.app.delete('/tokens/junk', expect_errors=True)
        self.assertIn('invalid secret',
                      resp.json['errors'][0]['description'])
        self.assertEqual(400, resp.status_code)

    def test_invalidate_token(self):
        username = 'BUILD-9876'
        token = self.app.post_json('/tokens', {'username': username})
        resp = self.app.delete(
            '/tokens/{id}'.format(id=token.json['secret']),
            params={'secret': token.json['secret']})
        self.assertEqual(200, resp.status_code)


class ApiTokenValidationTestCase(ApiBaseTestCase):

    def test_invalid_validate(self):
        resp = self.app.post_json(
            '/validate', {'junk': 'junk'}, expect_errors=True)
        self.assertEqual(400, resp.status_code)

    def test_validate_token(self):
        token = self.app.post_json('/tokens', {'username': 'BUILD-1234'})
        resp = self.app.post_json(
            '/validate', {'username': token.json['username'],
                          'secret': token.json['secret']})
        self.assertEqual(200, resp.status_code)

    def test_invalid_username(self):
        token = self.app.post_json('/tokens', {'username': 'BUILD-1234'})
        resp = self.app.post_json(
            '/validate', {'username': 'BUILD-INVALID',
                          'secret': token.json['secret']},
            expect_errors=True)
        self.assertEqual(404, resp.status_code)

    def test_invalid_secret(self):
        token = self.app.post_json('/tokens', {'username': 'BUILD-1234'})
        resp = self.app.post_json(
            '/validate', {'username': token.json['username'],
                          'secret': 'junk'},
            expect_errors=True)
        self.assertEqual(404, resp.status_code)


if __name__ == '__main__':
    unittest.main()
